!function(){"use strict";jQuery(document).ready((function(t){t(".cioFilterFoldable").each((function(){var e=t(this),i=e.find(".cioFilterToggle");i.removeClass("cioHidden"),t.chrysalio.state[e.attr("id")]||(e.addClass("cioFilterFold"),i.attr("src",i.attr("src").replace("minus","plus"))),i.click((function(){e.toggleClass("cioFilterFold"),t.chrysalio.state[e.attr("id")]=!e.hasClass("cioFilterFold"),t.chrysalio.saveState(),t.chrysalio.state[e.attr("id")]?i.attr("src",i.attr("src").replace("plus","minus")):i.attr("src",i.attr("src").replace("minus","plus"))}))})),t(".cioAutocomplete").each((function(){var e=t(this),i=window.location.pathname.replace(/\/$/,"")+"/filter",r=t(this).parent().parent().find(".cioFilterKey select").val(),a={};e.autocomplete({minLength:2,source:function(e,c){if(e.term in a)c(a[e.term]);else{var l={field:r,value:e.term};t.getJSON(i,l,(function(t,i,r){a[e.term]=t,c(t)}))}}})}))}))}();
