"""Unit, integration, and functional testing for LDAP module."""

from os.path import join, dirname

TEST1_LDAP_XML = join(dirname(__file__), 'Xml', 'test1.cioldap.xml')
TEST2_LDAP_XML = join(dirname(__file__), 'Xml', 'test2.cioldap.xml')
