"""CioSkeleton mode."""

from ...lib.i18n import _


# Mode = (mode_id, (icon, label, permission, route, menu))
MODE_CIOSKELETON = ('skeleton', (
    '{theme}/cioskeleton/images/menu_cioskeleton.png', _('Skeleton'),
    'mode-skeleton', 'bone_index', None))
