"""CIOSkeleton menu."""

from ...lib.i18n import _


# Menu entry = (icon, label, permission, route, (subentry, ...))
MENU_CIOSKELETON = (
    '{theme}/cioskeleton/images/menu_cioskeleton.png', _('Skeleton'),
    'bone-view', 'bone_index', None)
