var Cookies=function(){"use strict";var e,r,n,t=(e=function(e,r){var n;n=function(){function e(){for(var e=0,r={};e<arguments.length;e++){var n=arguments[e];for(var t in n)r[t]=n[t]}return r}function r(e){return e.replace(/(%[0-9A-Z]{2})+/g,decodeURIComponent)}return function n(t){function o(){}function i(r,n,i){if("undefined"!=typeof document){"number"==typeof(i=e({path:"/"},o.defaults,i)).expires&&(i.expires=new Date(1*new Date+864e5*i.expires)),i.expires=i.expires?i.expires.toUTCString():"";try{var c=JSON.stringify(n);/^[\{\[]/.test(c)&&(n=c)}catch(e){}n=t.write?t.write(n,r):encodeURIComponent(String(n)).replace(/%(23|24|26|2B|3A|3C|3E|3D|2F|3F|40|5B|5D|5E|60|7B|7D|7C)/g,decodeURIComponent),r=encodeURIComponent(String(r)).replace(/%(23|24|26|2B|5E|60|7C)/g,decodeURIComponent).replace(/[\(\)]/g,escape);var u="";for(var p in i)i[p]&&(u+="; "+p,!0!==i[p]&&(u+="="+i[p].split(";")[0]));return document.cookie=r+"="+n+u}}function c(e,n){if("undefined"!=typeof document){for(var o={},i=document.cookie?document.cookie.split("; "):[],c=0;c<i.length;c++){var u=i[c].split("="),p=u.slice(1).join("=");n||'"'!==p.charAt(0)||(p=p.slice(1,-1));try{var a=r(u[0]);if(p=(t.read||t)(p,a)||r(p),n)try{p=JSON.parse(p)}catch(e){}if(o[a]=p,e===a)break}catch(e){}}return e?o[e]:o}}return o.set=i,o.get=function(e){return c(e,!1)},o.getJSON=function(e){return c(e,!0)},o.remove=function(r,n){i(r,"",e(n,{expires:-1}))},o.defaults={},o.withConverter=n,o}((function(){}))},e.exports=n()},e(n={path:r,exports:{},require:function(e,r){return function(){throw new Error("Dynamic requires are not currently supported by @rollup/plugin-commonjs")}(null==r&&n.path)}},n.exports),n.exports);return t}();
